define(['./shared'], (function (actor) { 'use strict';

const noManager = (managerId) => ({
    cancel() { },
    value: Promise.reject(new Error(`No manager registered for ${managerId}`)),
});
/**
 * Receives messages from an actor in the web worker.
 */
class WorkerDispatch {
    /** There is one worker shared between all managers in the main thread using the plugin, so need to store each of their configurations. */
    managers = {};
    init = (message) => {
        this.managers[message.managerId] = new actor.LocalDemManager(message.demUrlPattern, message.cacheSize, message.encoding, message.maxzoom, message.timeoutMs);
        return { cancel() { }, value: Promise.resolve() };
    };
    fetchTile = (managerId, z, x, y, timer) => this.managers[managerId]?.fetchTile(z, x, y, timer) || noManager(managerId);
    fetchAndParseTile = (managerId, z, x, y, timer) => actor.prepareDemTile(this.managers[managerId]?.fetchAndParseTile(z, x, y, timer) ||
        noManager(managerId), true);
    fetchContourTile = (managerId, z, x, y, options, timer) => actor.prepareContourTile(this.managers[managerId]?.fetchContourTile(z, x, y, options, timer) ||
        noManager(managerId));
}

// eslint-disable-next-line no-restricted-globals
const g = typeof self !== "undefined" ? self : window;
g.actor = new actor.Actor(g, new WorkerDispatch());

}));
